﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Utility : MonoBehaviour
{
	public static Vector3 copyVector( Vector3 vector )
	{
		return new Vector3( vector.x, vector.y, vector.z );
	}

	public static float easeIn( float from, float to, float t )
	{
		return Mathf.Lerp( from, to, 1.0f - Mathf.Cos( t * Mathf.PI * 0.5f ) );
	}

	public static float easeOut( float from, float to, float t )
	{
		return Mathf.Lerp( from, to, Mathf.Sin( t * Mathf.PI * 0.5f ) );
	}

	public static float easeBellCurve( float from, float to, float t )
	{
		return Mathf.Lerp( from, to, -0.5f * Mathf.Cos( 2f * Mathf.PI * t ) + 0.5f );
	}

	public static Vector3 smoothStep( Vector3 from, Vector3 to, float t )
	{
		return new Vector3( Mathf.SmoothStep( from.x, to.x, t ),
			Mathf.SmoothStep( from.y, to.y, t ),
			Mathf.SmoothStep( from.z, to.z, t ) );
	}

	public static float directionToAngle( Vector2 dir )
	{
		float angle = Mathf.Atan2( -dir.x, dir.y ) * Mathf.Rad2Deg;

		if ( angle < 0 )
			angle += 360.0f;

		return angle;
	}

	public static Vector3 directionToEuler( Vector3 direction )
	{
		if ( direction == Vector3.zero )
			return Vector3.zero;

		return Quaternion.LookRotation( direction ).eulerAngles;
	}

	public static float ratio( float value, float min, float max )
	{
		float distance = Mathf.Abs( max - min );
		return ( value - min ) / distance;
	}

	public static float ratioClamped( float value, float min, float max )
	{
		float distance = Mathf.Abs( max - min );
		return Mathf.Clamp( ( value - min ) / distance, 0f, 1f );
	}

	public static float clampAngle( float value )
	{
		float angle = value % 360.0f;
		if ( angle < 0.0f )
			angle += 360.0f;
		return angle;
	}

	public static float clampAngleOffset( float value )
	{
		float angle = clampAngle( value );
		if ( angle > 180.0f )
			angle -= 360.0f;
		return angle;
	}

	public static Vector3 direction( Vector3 from, Vector3 to )
	{
		Vector3 diff = to - from;
		diff.Normalize();
		return diff;
	}

	public static Vector3 angleToDirection( float angle )
	{
		return new Vector3( Mathf.Sin( Mathf.Deg2Rad * angle ), 0.0f, Mathf.Cos( Mathf.Deg2Rad * angle ) );
	}

	public static Vector3 eulerToDirection( float pitch, float yaw )
	{
		pitch *= Mathf.Deg2Rad;
		yaw *= Mathf.Deg2Rad;
		return new Vector3( Mathf.Sin( yaw ) * Mathf.Cos( pitch ),
			Mathf.Sin( pitch ),
			Mathf.Cos( yaw ) * Mathf.Cos( pitch ) );
	}

	/// <summary>
	/// Yaw from position A to position B
	/// </summary>
	/// <param name="a">From position</param>
	/// <param name="b">To position</param>
	/// <returns></returns>
	public static float directionYaw( Vector3 from, Vector3 to )
	{
		float diffX = to.x - from.x;
		float diffZ = to.z - from.z;
		float angle = Mathf.Atan2( diffX, diffZ ) * Mathf.Rad2Deg;
		if ( angle < 0.0f )
			angle += 360.0f;
		return angle;
	}

	public static float angleBetween( Vector2 from, Vector2 to )
	{
		Vector2 difference = to - from;
		float angle = Mathf.Atan2( difference.x, difference.y ) * Mathf.Rad2Deg;

		if ( angle < 0.0f )
			angle += 360.0f;

		return angle;
	}

	public static float angleBetweenDirections( Vector3 from, Vector3 to )
	{
		return Vector3.Angle( from, to );
	}

	public static bool angleInCone( float angle, float coneLeftAngle, float coneRightAngle )
	{
		float _angle = clampAngle( angle );
		float _coneLeft = clampAngle( coneLeftAngle );
		float _coneRight = clampAngle( coneRightAngle );
		float range = clampAngle( _coneRight - _coneLeft );
		float angleRange = clampAngle( _coneRight - _angle );

		if ( range == 0.0f )
			return true;

		return ( angleRange >= 0.0f && angleRange <= range );
	}

	public static Vector3 projection( Vector3 start, float pitch, float yaw, float distance )
	{
		return start + ( eulerToDirection( pitch, yaw ) * distance );
	}

	public static Vector3 projection( Vector3 start, Vector3 direction, float distance )
	{
		return start + ( direction * distance );
	}

	public static Vector3 projectionTranslate( Vector3 position, Vector3 forward, Vector3 offset )
	{
		Vector3 right = -Vector3.Cross( forward.normalized, Vector3.up ).normalized;
		Vector3 up = Vector3.Cross( -right, forward.normalized ).normalized;
		return position + ( up * offset.y ) + ( right * offset.x ) + ( forward * offset.z );
	}

	public static Vector3 lineClosestPoint( Vector3 a, Vector3 b, Vector3 position )
	{
		Vector3 v1 = position - a;
		Vector3 v2 = ( b - a ).normalized;
		float d = Vector3.Distance( a, b );
		float t = Vector3.Dot( v2, v1 );
		if ( t <= 0f )
			return a;
		if ( t >= d )
			return b;
		return a + ( v2 * t );
	}

	public static string randString( int length )
	{
		string characters = "!@#$%^&*()_-=+[]{}:;,<.>/?0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
		string hash = "";

		for ( int i = 0; i < length; ++i )
			hash += characters[Random.Range( 0, characters.Length - 1 )];

		return hash;
	}

	public static string randHash( int length )
	{
		string characters = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
		string hash = "";

		for ( int i = 0; i < length; ++i )
			hash += characters[Random.Range( 0, characters.Length - 1 )];

		return hash;
	}

	public static string md5( string value )
	{
		// http://wiki.unity3d.com/index.php?title=MD5
		System.Text.UTF8Encoding ue = new System.Text.UTF8Encoding();
		byte[] bytes = ue.GetBytes( value );

		// encrypt bytes
		System.Security.Cryptography.MD5CryptoServiceProvider md5 = new System.Security.Cryptography.MD5CryptoServiceProvider();
		byte[] hashBytes = md5.ComputeHash( bytes );

		// Convert the encrypted bytes back to a string (base 16)
		string hashString = "";

		for ( int i = 0; i < hashBytes.Length; i++ )
		{
			hashString += System.Convert.ToString( hashBytes[i], 16 ).PadLeft( 2, '0' );
		}

		return hashString.PadLeft( 32, '0' );
	}
}
